@@ -1,5 +1,5 @@
 #******************************************************************************
-# $Id: POP3Client.pm,v 2.15 2003/10/18 23:12:46 ssd Exp $
+# $Id: POP3Client.pm,v 2.17 2005/08/18 15:28:12 ssd Exp $
 #
 # Description:  POP3Client module - acts as interface to POP3 server
 # Author:       Sean Dowd <pop3client@dowds.net>
@@ -13,6 +13,7 @@
 package Mail::POP3Client;
 
 use strict;
+use warnings;
 use Carp;
 use IO::Socket qw(SOCK_STREAM);
 
@@ -28,8 +29,8 @@ require Exporter;
 
 );
 
-my $ID =q( $Id: POP3Client.pm,v 2.15 2003/10/18 23:12:46 ssd Exp $ );
-$VERSION = substr q$Revision: 2.15 $, 10;
+my $ID =q( $Id: POP3Client.pm,v 2.17 2005/08/18 15:28:12 ssd Exp $ );
+$VERSION = substr q$Revision: 2.17 $, 10;
 
 
 # Preloaded methods go here.
@@ -324,11 +325,16 @@ sub Close
 	$me->Message("Socket read failed for QUIT");
 	# XXX: Should add the following?
 	#$me->State('DEAD');
+	undef $me->{SOCKET};
 	return 0;
     }
     $me->Message( $line );
-    close( $me->Socket() ) or $me->Message("close failed: $!") and return 0;
+    close( $me->Socket() ) or $me->Message("close failed: $!") and do {
+      undef $me->{SOCKET};
+      return 0;
+    };
     $me->State('DEAD');
+    undef $me->{SOCKET};
     $line =~ /^\+OK/i || return 0;
   }
   1;
@@ -366,6 +372,7 @@ sub Connect
         eval {
 	  require IO::Socket::SSL;
 	};
+      $@ and $me->Message("Could not load IO::Socket::SSL: $@") and return 0;
       $s = IO::Socket::SSL->new( PeerAddr => $me->Host(),
 				 PeerPort => $me->Port(),
 				 Proto    => "tcp",
@@ -461,7 +468,6 @@ sub login { Login(@_); }
 sub Login_APOP
 {
   my $me = shift;
-
   eval {
     require Digest::MD5;
   };
@@ -479,6 +485,14 @@ sub Login_APOP
   }
   chomp $line;
   $me->Message($line);
+  # some servers will close here...
+  $me->NOOP() || do {
+    $me->State('DEAD');
+    undef $me->{SOCKET};
+    $me->Message("APOP failed: server has closed the socket");
+    return 0;
+  };
+
   $line =~ /^\+OK/ or $me->Message("APOP failed: $line") and return 0;
   $me->State('TRANSACTION');
 
@@ -711,7 +725,9 @@ sub Body
 	$me->Message("Socket read failed for RETR");
 	return;
     }
-  } until $line =~ /^\s*$/;
+    $line =~ s/[\r\n]//g;
+  } until $line =~ /^(\s*|\.)$/;
+  $line =~ /^\.\s*$/ && return;  # we found a header only!  Lotus Notes seems to do this.
 
   while (1) {
     $line = $me->_sockread();
@@ -759,7 +775,9 @@ sub BodyToFile
 	$me->Message("Socket read failed for RETR");
 	return;
     }
-  } until $line =~ /^\s*$/;
+    $line =~ s/[\r\n]//g;
+  } until $line =~ /^(\s*|\.)$/;
+  $line =~ /^\.\s*$/ && return;  # we found a header only!  Lotus Notes seems to do this.
 
   while (1) {
     $line = $me->_sockread();
@@ -1042,6 +1060,22 @@ sub Xtnd {
   return $line;
 }
 
+#******************************************************************************
+#* NOOP - used to check socket
+#******************************************************************************
+sub NOOP {
+  my $me = shift;
+
+  my $s = $me->Socket();
+  $me->Alive() or return 0;
+ 
+  $me->_sockprint( "NOOP", $me->EOL );
+  my $line = $me->_sockread();
+#  defined( $line ) or return 0;
+  $line =~ /^\+OK/ or return 0;
+  return 1;
+}
+
 
 #******************************************************************************
 #* Mail::IMAPClient compatibility functions (wsnyder@wsnyder.org)
@@ -1294,7 +1328,7 @@ it is not set or is 110.  Otherwise, it will use your port.  If USESSL
 is true, IO::Socket::SSL will be loaded.  If it is not in your perl,
 the call to connect will fail.
 
-new returns a valid Mail::POP3CLient object in all cases.  To test for
+new returns a valid Mail::POP3Client object in all cases.  To test for
 a connection failure, you will need to check the number of messages:
 -1 indicates a connection error.  This will likely change sometime in
 the future to return undef on an error, setting $! as a side effect.
@@ -1367,10 +1401,10 @@ QUIT (invoking the Close method).  Can be reset with a Reset message.
 =item I<Connect>
 
 Start the connection to the POP3 server.  You can pass in the host and
-port.  Returns the number of messages in the mailbox, or -1 on a
-connection error.  The constructors return the value of Connect, so
-they will never return undef.  This will change in any version 3.x
-release, but never in a 2.x release.
+port.  Returns 1 if the connection succeeds, or 0 if it fails (Message
+will contain a reason).  The constructor always returns a blessed
+reference to a Mail::POP3Client obhect.  This may change in a version
+3.x release, but never in a 2.x release.
 
 =item I<Close>
 
